//
//  ViewController.m
//  SocialLocation
//
//  Created by Devon on 2012-12-10.
//  Copyright (c) 2012 Devon. All rights reserved.
//

#import <Accounts/Accounts.h>
#import <Social/Social.h>
#import "ViewController.h"

#define DEFAULT_TWITTER_ACCOUNT_KEY             @"com.defaultTwitterAccount.SocialLocation"
#define MAX_FACEBOOK_TOKEN_RENEWAL_ATTEMPTS     3

@interface ViewController () <UIActionSheetDelegate> {
    NSMutableArray *_arrayOfACAccounts;
    NSInteger _actionSheetCancelButtonIndex;
}
@property (strong, nonatomic) NSString *locationInfo;
@property (weak, nonatomic) IBOutlet UIImageView *imageView;

// AccountStore needs to live for as long as accounts live
@property (strong, nonatomic) ACAccountStore *accountStore;
@property (strong, nonatomic) ACAccount *account;
@end

@implementation ViewController

- (ACAccountStore *)accountStore
{
    if (!_accountStore) {
        self.accountStore = [[ACAccountStore alloc] init];
    }

    return _accountStore;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    _arrayOfACAccounts = [[NSMutableArray alloc] init];

    // Get current location without using location services
    NSString *countryCode = [[NSLocale currentLocale] objectForKey:NSLocaleCountryCode];
    NSString *countryName = [[NSLocale currentLocale] displayNameForKey:NSLocaleCountryCode value:countryCode];
    NSTimeZone *timezone = [NSTimeZone localTimeZone];

    NSString *location = [NSString stringWithFormat:@"Current Locale: %@\nCountry Name: %@\nCurrent Time Zone: %@", countryCode, countryName, timezone.name];
    self.locationInfo = location;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Social Account Selection

- (IBAction)shareButtonTapped:(id)sender
{
    NSArray *activityItems;

    // If there is an image, share image too
    if (_imageView.image != nil) {
        activityItems = @[ _locationInfo, _imageView.image];
    } else {
        activityItems = @[ _locationInfo ];
    }

    UIActivityViewController *activityController = [[UIActivityViewController alloc] initWithActivityItems:activityItems applicationActivities:nil];
    [self presentViewController:activityController animated:YES completion:nil];
}

- (void)postToSocialPlatform:(NSString *)platform completion:(void (^)(BOOL success))completion
{
    SLComposeViewController *composerSheet = [[SLComposeViewController alloc] init];
    // Check if Facebook account is available first
    // +isAvailableForServiceType always returns YES in simulator
    if ([SLComposeViewController isAvailableForServiceType:platform]) {
        composerSheet = [[SLComposeViewController alloc] init];
        // Which social plattform to use
        composerSheet = [SLComposeViewController composeViewControllerForServiceType:platform];
        // The message to post/tweet
        [composerSheet setInitialText:[NSString stringWithFormat:_locationInfo, composerSheet.serviceType]];
        // Specify what image to post/tweet
        [composerSheet addImage:_imageView.image];
        // To include link or other content, refer to https://developer.apple.com/library/ios/#documentation/NetworkingInternet/Reference/SLComposeViewController_Class/Reference/Reference.html#//apple_ref/doc/uid/TP40012205
        [self presentViewController:composerSheet animated:YES completion:nil];
    } else {
        NSString *platformName;
        if ([platform isEqualToString:SLServiceTypeFacebook]) {
            platformName = NSLocalizedString(@"Facebook", nil);
        } else if ([platform isEqualToString:SLServiceTypeTwitter]) {
            platformName = NSLocalizedString(@"Twitter", nil);
        }

        NSString *errorMessage = [NSString stringWithFormat:NSLocalizedString(@"%@ is unavailable. Please set up %@ in Settings.", nil), platformName, platformName];
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Service Unavailable", nil)
                                                        message:errorMessage
                                                       delegate:nil
                                              cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                              otherButtonTitles:nil];
        [alert show];
    }

    composerSheet.completionHandler = ^(SLComposeViewControllerResult result) {
        switch (result) {
            case SLComposeViewControllerResultCancelled:
                NSLog(@"Cancelled");
                break;
            case SLComposeViewControllerResultDone:
                NSLog(@"Successful");
                break;
            default:
                break;
        }

        if (completion) {
            completion(result);
        }
    };
}

- (IBAction)shareToFacebookButtonTapped:(id)sender
{
    [self postToSocialPlatform:SLServiceTypeFacebook completion:nil];
}

- (IBAction)shareToTwitterButtonTapped:(id)sender
{
    [self postToSocialPlatform:SLServiceTypeTwitter completion:^(BOOL success) {
        if (success == SLComposeViewControllerResultCancelled) {
            [self dismissViewControllerAnimated:YES completion:nil];
        }
    }];
}

- (IBAction)tweetButtonTapped:(id)sender
{
    // Prepare accountType. Account type can be Facebook or Twitter;
    ACAccountType *accountType = [self.accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];

    // Prepare a weak pointer to self to be used in completion handler
    __weak ViewController *weakself = self;

    // Request access to user's iOS Facebook/Twitter account
    [self.accountStore requestAccessToAccountsWithType:accountType options:nil
                                            completion:^(BOOL granted, NSError *error) {
                                                if (granted) {
                                                    NSArray *accounts = [weakself.accountStore accountsWithAccountType:accountType];

                                                    // Get default Twitter account name from user default in the case of multiple accounts
                                                    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
                                                    NSString *defaultAccountName = [defaults objectForKey:DEFAULT_TWITTER_ACCOUNT_KEY];

                                                    if (!defaultAccountName) {
                                                        // If no default account stored in user default yet...
                                                        if (accounts.count == 1) {
                                                            // Only 1 iOS Twitter account; save it to user default
                                                            NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
                                                            ACAccount *account = [accounts lastObject];
                                                            [defaults setObject:account.username forKey:DEFAULT_TWITTER_ACCOUNT_KEY];
                                                            [defaults synchronize];

                                                            // Tweet
                                                            [weakself tweet];
                                                        }
                                                        else {
                                                            // More than 1 iOS Twitter Account; ask user which one they would like to use
                                                            UIActionSheet *sheet = [[UIActionSheet alloc] initWithTitle:NSLocalizedString(@"Pick a Twitter account you would like to post tweets to", nil)
                                                                                                               delegate:self
                                                                                                      cancelButtonTitle:nil
                                                                                                 destructiveButtonTitle:nil
                                                                                                      otherButtonTitles:nil];

                                                            // Add account names to sheet and store in an array
                                                            [accounts enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                                                                ACAccount *account = obj;
                                                                [_arrayOfACAccounts addObject:account.username];
                                                                [sheet addButtonWithTitle:account.username];
                                                            }];

                                                            // Add cancel button
                                                            sheet.cancelButtonIndex = [sheet addButtonWithTitle:NSLocalizedString(@"Cancel", nil)];

                                                            // Save cancel button index to be accessed by delegate
                                                            _actionSheetCancelButtonIndex = sheet.cancelButtonIndex;

                                                            // Show sheet in main queue
                                                            [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                                                                [sheet showInView:weakself.view];
                                                            }];

                                                            // Tweeting will be handled by UIActionSheetDelegate in this case
                                                        }
                                                    }
                                                    else {
                                                        // has default account
                                                        [weakself tweet];
                                                    }
                                                }
                                                else {
                                                    // Permission denied
                                                    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                                                        [weakself errorAccessingSocialAccounts:error];
                                                    }];
                                                }
                                            }];
}

#pragma mark - Posting Tweet

- (void)tweet
{
    void (^showAlert)(NSString *, NSString *) = ^(NSString *title, NSString *message) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title
                                                        message:message
                                                       delegate:nil
                                              cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                              otherButtonTitles:nil];
        [alert show];
    };

    __weak ViewController *weakself = self;

    // Get iOS Twitter account
    ACAccountType *accountType = [self.accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];
    NSArray *accounts = [self.accountStore accountsWithAccountType:accountType];
    NSString *defaultTwitterAccount = [[NSUserDefaults standardUserDefaults] objectForKey:DEFAULT_TWITTER_ACCOUNT_KEY];

    // At this time there should definitely be an account stored in user default; otherwise, something is very wrong.
    if (defaultTwitterAccount) {
        [accounts enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            ACAccount *twitterAccount = obj;
            if ([twitterAccount.username isEqualToString:defaultTwitterAccount]) {
                weakself.account = twitterAccount;
                *stop = YES;
            }
        }];

        if (weakself.account) {
            NSDictionary *message = @{ @"status": self.locationInfo };
            NSURL *requestURL = [NSURL URLWithString:@"https://api.twitter.com/1/statuses/update.json"];
            SLRequest *postRequest = [SLRequest requestForServiceType:SLServiceTypeTwitter
                                                        requestMethod:SLRequestMethodPOST
                                                                  URL:requestURL
                                                           parameters:message];

            postRequest.account = self.account;
            [postRequest performRequestWithHandler:^(NSData *responseData, NSHTTPURLResponse *urlResponse, NSError *error) {
                if (error) {
                    // Must send any UIView and subclass work back to main thread
                    NSLog(@"Twitter HTTP response: %i", [urlResponse statusCode]);
                    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                        NSString *errorMessage = [NSString stringWithFormat:@"%@%i", NSLocalizedString(@"Failed posting tweet. Twitter HTTP response: ", nil), [urlResponse statusCode]];
                        showAlert(NSLocalizedString(@"Error", nil), errorMessage);
                    }];
                } else {
                    [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                        showAlert(NSLocalizedString(@"Success!", nil), NSLocalizedString(@"Your tweet has been successfully posted", nil));
                    }];
                }
            }];
        } else {
            // It's possible that user goes into Settings and makes change to Twitter accounts which results in
            // the account stored in user default being out of sync with what's really on the device. Handle the situation here
        }
    }
}

- (IBAction)postLocInfoToFacebookStatus:(id)sender
{
    ACAccountType *facebookAccountType = [self.accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierFacebook];

    NSDictionary *facebookAccessDictionary = @{
        ACFacebookAppIdKey: @"<enter your own id>",             // This parameter is for the Facebook AppId
        ACFacebookPermissionsKey: @[ @"publish_stream" ],   // What you want to access; in this case, publish_steam is the feed
        ACFacebookAudienceKey: ACFacebookAudienceFriends    // Who can see your post
    };

    __weak ViewController *weakself = self;
    [self.accountStore requestAccessToAccountsWithType:facebookAccountType
                                               options:facebookAccessDictionary
                                            completion:^(BOOL granted, NSError *error) {
                                                if (granted) {
                                                    NSArray *accounts = [weakself.accountStore accountsWithAccountType:facebookAccountType];
                                                    // There can be only 1 Facebook account in iOS 6
                                                    weakself.account = [accounts lastObject];
                                                    [weakself postLocationInfoToFacebookFeed];
                                                } else {
                                                    // Permission denied
                                                    dispatch_async(dispatch_get_main_queue(), ^{
                                                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Perission Denied", nil)
                                                                                                        message:NSLocalizedString(@"We do not have access to your Facebook account.", nil)
                                                                                                       delegate:nil
                                                                                              cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                                                                              otherButtonTitles:nil];
                                                        [alert show];
                                                    });
                                                }
                                            }];
}

- (void)postLocationInfoToFacebookFeed
{
    NSAssert(self.account != nil, @"Must have a valid account");

    NSDictionary *parameters = @{ @"message": self.locationInfo };
    NSURL *feedURL = [NSURL URLWithString:@"https://graph.facebook.com/me/feed"];

    SLRequest *request = [SLRequest requestForServiceType:SLServiceTypeFacebook
                                            requestMethod:SLRequestMethodPOST
                                                      URL:feedURL
                                               parameters:parameters];

    // Associate account with request
    request.account = self.account;

    // Dispatch request
    [request performRequestWithHandler:^(NSData *responseData, NSHTTPURLResponse *urlResponse, NSError *error) {
        if (error) {
            // Handle error
            NSLog(@"%i Posting to Facebook feed failed: %@", [urlResponse statusCode], [error userInfo]);
        } else {
            // Inform user that status has been posted
            NSLog(@"Location info has been posted");
        }
    }];
}

#pragma mark - UIActionSheetDelegate

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex < _arrayOfACAccounts.count) {
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        [defaults setObject:_arrayOfACAccounts[buttonIndex] forKey:DEFAULT_TWITTER_ACCOUNT_KEY];
        [defaults synchronize];

        // Tweet
        [self tweet];
    }
}

#pragma mark - Helper Method

- (void)errorAccessingSocialAccounts:(NSError *)error
{
    // Check if Twitter is available but user denied access to her account
    BOOL canSendTweet = [SLComposeViewController isAvailableForServiceType:SLServiceTypeTwitter];

    NSString *errorMessage;
    if (canSendTweet && !error) {
        // Has account in db, but permission to access Twitter account is rejected
        errorMessage = NSLocalizedString(@"We do not have permission to access your Twitter account. You can allow accessing to your Twitter account at any time in Settings > Privacy > Twitter, and grant access to this app by switching on.", nil);
    } else if (!canSendTweet && !error) {
        // Has account in db, but encounters error
        errorMessage = NSLocalizedString(@"Unable to access your Twitter account at this time. Please try again later.", nil);

    } else {
        // No Twitter account in Settings
        errorMessage = NSLocalizedString(@"We are unable to find Twitter account on your phone. You can set it up in Settings > Twitter. Please try again after you have set up your Twitter account.", nil);
    }
    
    // Show alert
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:nil
                                                    message:errorMessage
                                                   delegate:nil
                                          cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                          otherButtonTitles:nil];
    
    [alert show];
}

@end
